
/**
 * The Customer class represents a customer in a rental car system.
 * It holds a subset of the customer details relevant to the 
 * rental car business.
 * 
 * @author Bernd J. Krmer
 * @version 22.12.2003
 */
public class Customer
{
    // static counter variable for creating unique customer IDs
    private static int previousCustomerNo = 0;
    
    // the customer's full name
    private String name;
    // the credit card number of this customer
    private int creditCardNo;
    // the customerID is composed of the first letter 
    // of the customer's name and the current value 
    // of class variable previousCustomerNo
    private String customerID;
    // a prefix that indicates an invalid customerID
    private String undefined = "undefined";

    /**
     * Create a new customer with a given name.
     * @param  fullName   a non-empty string 
     * @param  customersCreditcardNo   an int value between 500 and 599
     */
    public Customer(String fullName, int customersCreditcardNo)
    {
        if (!checkName(fullName)) // replaces previous empty string check
        { 
           System.out.println("Illegal Name!");
           name = undefined;
           customerID = undefined + ++previousCustomerNo;
        } else
        {
            name = normalizeName(fullName); // normalization has been added here
            customerID = name.substring(0,1) + ++previousCustomerNo;
        }
        setCreditCardNo(customersCreditcardNo);
    }
    

    /**
     * Return the full name of this customer.
     * 
     * @return the name attribute
     */
    public String getName()
    {
        return name;
    }

    /**
     * Redefine the creditcard number of this customer.
     * customersCreditcardNo
     * 
     * @param  customersCreditcardNo   an int value between 500 and 599
     */
    public void setCreditCardNo(int customersCreditcardNo)
    {
        if (customersCreditcardNo < 500 || customersCreditcardNo > 599)
        {
            System.out.println("Illegal number! Please provide legal number " +
                               "between 500 and 599!"); 
        } else 
        { 
            creditCardNo = customersCreditcardNo; 
        }            
    }
    
    /**
     * Return the creditcard number of this customer.
     * 
     * @return the creditCardNo attribute
     */
    public int getCreditCardNo()
    {
        return creditCardNo;
    }

    /**
     * Return the internal ID of this customer. 
     * 
     * @return customerID 
     */
    public String getCustomerId()
    {
        return customerID;
    }
    
    /**
     * Print the customers's name and credit card number to the output terminal.
     */
    public void print()
    {
        System.out.println(name + " (" + creditCardNo 
                    + ", " + getCustomerId() + ")");
    }

    /**
     * Checks whether a name starts with a letter and is followed by zero 
     * or more letters, hyphens or space characters.
     * 
     * @param  String denoting a customer name
     * @return true if is legal name, otherwise false.
     */    
    private boolean checkName(String name)
    {
        return !isEmptyString(name) && firstIsChar(name) && restIsNameChar(name);
    }

    /**
     * Sets the first letter of a legar name to upper case and 
     * the rest of the name to lowercase letters.
     * 
     * @param  String denoting a legal name
     * @return normalized name string
     */        
    private String normalizeName(String name)
    {
        return (name.substring(0,1).toUpperCase() +
                    name.substring(1,name.length()).toLowerCase());
    }
        
    private boolean isLetter(char c)
    {
        return ((int) c > 64 && (int) c < 91) ||
               ((int) c > 96) && ((int) c < 122);
    }
    
    private boolean isNameChar(char c)
    {
        return isLetter(c) || isSpace(c) || isHypen(c);
    }    

    private boolean isHypen(char c)
    {
        return (int) c == 45;
    }
    
    private boolean isSpace(char c)
    {
        return (int) c == 32;
    }
    
    private boolean isEmptyString(String name)
    {
        return name.length() == 0;
    }
    
    private boolean firstIsChar(String name)
    {
        return isLetter(name.charAt(0));
    }   
    
    private boolean restIsNameChar(String name)
    {
        boolean yes = true;
        for (int i = 1; i< name.length(); i++)
        {
            if (!isNameChar(name.charAt(i)))
            {
                yes = false;
                break;
            };
        }
        return yes;
    }   
}
